'
'----------------------     Fuer Pollin Display 16080 mit Touch      ------------------------
'--------------------------------------------------------------------------------------------
'
'
' Alex D. (allu)   9/2009
'
' Hinweis:
'         - Wenn Fehler zu ADCSRA oder ADCSR gemeldet werden, den jeweils
'         .....anderen Registernamen einsetzen.
'
' Angepasst an Mega32, 16/11/2009
'           , , , a, , ,  in den Charaktergenerator eingebaut,
'           .....gem ASCII-Zeichensatz, deutsche Variante.
'
' Hinweis zum Mega32:
'         - JTAG-Interface des Mega32 muss ausgeschaltet sein,
'         .....ansonsten knnen PC2 bis PC4 nicht verwendet werden
'
' Hinweis zu UART und PD0 und PD1
'         - Falls die Baudrate fr die serielle Schnittstelle initialisiert wurde,
'         ..... . sind PD0 und PD1 nicht mehr verfgbar
'
' Gendert: 17/11/2009
'           - Freie Port und Bit Wahl fr den Control-Bus zum Display eingebaut.
'           - Hardware-Anpassungen in einem Block zusammemgefasst
'           - Display-Ansteuerungs-Zeiten fr 16 Mhz angepasst
'
' Gendert: 18/11/2009
'           - Ohm- und C-Symbol (Grad Celsius) ergnzt.
'
' Gendert: 20/11/2009
'           - Programmteile ausgalagert,
'           ..... aufgeteilt in RomTabs.txt, LCD16080.txt, Touch16080.txt
'           ..... Initialisierung
'
'
'---------------------------  Information zu einzelnen Funktionen ---------------------------
'--------------------------------------------------------------------------------------------
'
' Locate_zeile = 31                               Schreibe Text auf Zeile 31
' Locate_spalte = 0                               Schreibe Text ab Spalte 0
' Muster    = "01234567890123456789012"           Muster dient als Voralge fr die Gestaltung
' Lcd_string ="Die ist ein Testext    "           Darstellung eines Textes auf dem Display
' Gosub Lcd_write_string                          Textstring verarbeiten und ausgeben
' Lcd_form_3 = "(7        )  (4       )"          Touch reagiert auf Flchen innerhalt der Klammern
'
' weiter mit Beschreibungen.....
'


'------------------------------------------------------------------
'Konfiguration der Mikrocontrollerumgebung
'------------------------------------------------------------------

$regfile = "m32def.DAT"                                     ' ATMega32
'$regfile = "m8def.DAT"                                      ' ATMega8

'$crystal = 8000000                                          ' Anpassen der Taktfrequenz
$crystal = 16000000                                         ' 16MHz

' Config Com1 = 19200 , Synchrone = 0 , Parity = None , Stopbits = 1 , Databits = 9 , Clockpol = 0

                     ' Reserviert TxD und RxD fr die serielle Schnittstelle
                     ' PD0 und PD1 sind dann nicht mehr verfgbar

$hwstack = 128
$swstack = 256
$framesize = 128


'------------------------------------------------------------------
'PortPins zum Display 16080 festlegen und initialisieren
'------------------------------------------------------------------

'------------------------------------------------------------------
' Daten-Bus zum Display (hier auf PortC)
'------------------------------------------------------------------
Daten_bus_out Alias Portc
Daten_bus_in Alias Pinc
Daten_ddr Alias Ddrc

'------------------------------------------------------------------
' Steuersignale zum Display (hier auf PortD)
'------------------------------------------------------------------
Control_bus_rs Alias Portd
Control_bus_rs_ddr Alias Ddrd
Rs Alias 6                                                  'RS Pin Display

Control_bus_rw Alias Portd
Control_bus_rw_ddr Alias Ddrd
Rw Alias 5                                                  'R/W Pin Display

Control_bus_e Alias Portd
Control_bus_e_ddr Alias Ddrd
E Alias 7                                                   'Enable Pin Display

Control_bus_cs Alias Portd
Control_bus_cs_ddr Alias Ddrd
Cs Alias 4                                                  'Chipsel. Pin Display

Control_bus_r Alias Portd
Control_bus_r_ddr Alias Ddrd
R Alias 3                                                   'Reset Pin Display

'------------------------------------------------------------------
' AD-Wandler Touchanschluss (hier auf PortA)
'------------------------------------------------------------------
Touch_control Alias Porta
Touch_ddr Alias Ddra

'------------------------------------------------------------------
' ENDE Portanpassung
'------------------------------------------------------------------


'------------------------------------------------------------------
'      Daten-Bus und Controlbus zum Display initialisieren
'------------------------------------------------------------------
Gosub Display_ports_initialisieren



' und weiter mit dem Initialisieren
$include "Initialisierung.txt"


Dim Irq_teiler As Byte                                      ' Vorteiler fr Touchaufruf
Dim Ticker As Byte                                          ' Vorteiler fr die Timer
Dim Timer_a As Byte                                         ' Timer A = 0,1 bis 25,5 sec
Dim Timer_b As Byte                                         ' Timer B = 0,1 bis 25,5 sec


'------------------------------------------------------------------
'Verstndigung zwischen Bascom und Touch
'------------------------------------------------------------------

                                                             ' Touch meldet:
Dim Touch_meldet As Byte                                    ' Berhrung erkannt = ja = 1
Ja Alias 1                                                  ' ... 1 = Touchberhrung wurde erkannt
Erledigt Alias 0                                            ' ... 0-setzen fr Quittung = Bearbeitet

Dim Touch_aktiviert As Byte                                 ' Solange der Touch berhrt wird = 2, sonst 0

Dim X_aktuell As Byte                                       ' X-Feinwert 0 bis 255, Berhrungspunkt                                                   '
Dim Y_aktuell As Byte                                       ' Y-Feinwert 0 bis 255, Berhrungspunkt

Dim X_melden As Byte                                        ' X-Rasterwert 0 bis 22, Klalibriert
Dim Y_melden As Byte                                        ' Y-Rasterwert 0 bis 7, Klalibriert
Dim F_taste As Byte                                         ' Rckgabewert des berhrten Rasterskstchens


'------------------------------------------------------------------
'Verstndigung zwischen Bascom und Display
'------------------------------------------------------------------


' Der Inhalt von LCD-String wird mit Gosub LCD_Write ab der
' ..... Position Locate_Spalte und Locate_Zeile ins Display geschrieben
' ..... Achtung: keine berlaufbremse eingebaut, kein Zeilenumbruch
_
Dim Locate_zeile As Byte                                    ' 0 bis 79 = 80 Zeilen
Dim Locate_spalte As Byte                                   ' 0 bis 22 = 23 Spalten
Dim Lcd_string As String * 23                               ' wird ins Displays geschrieben


'------------------------------------------------------------------
' Vorbereitet fr Texteingabe ber Touch
'------------------------------------------------------------------

Dim Touch_string As String * 23
Dim Touch_string_zeiger As Byte


'------------------------------------------------------------------
' Tabelle zur Aufnahme der Rckgabewerte im Raster
' Gosub Lcd_form_clear lscht die Rckgabewerte
' Gosub Lcd_form_aufbau erzeugt die Funktiontasten
'------------------------------------------------------------------
Dim Lcd_format(192) As Byte
Dim Lcd_form_0 As String * 23 At Lcd_format(1) Overlay
Dim Lcd_form_1 As String * 23 At Lcd_format(25) Overlay
Dim Lcd_form_2 As String * 23 At Lcd_format(49) Overlay
Dim Lcd_form_3 As String * 23 At Lcd_format(73) Overlay
Dim Lcd_form_4 As String * 23 At Lcd_format(97) Overlay
Dim Lcd_form_5 As String * 23 At Lcd_format(121) Overlay
Dim Lcd_form_6 As String * 23 At Lcd_format(145) Overlay
Dim Lcd_form_7 As String * 23 At Lcd_format(169) Overlay


$eeprom
Touch_kal_werte:                                            ' Platzhalter und Defaultwerte

Data Kalibriert                                             ' schon kalibriert ?
Data 50                                                     ' X_1_Pos = rechter Randabstand
Data 60                                                     ' Y_1_Pos = linker Randabstand
Data 122                                                    ' X_pro_Spaltenbreite
Data 85                                                     ' Y_pro_Zeilenhhe

$data



Gosub Display_und_touch_initialisieren

'------------------------------------------------------------------
' Hier beginnt das Hauptprogramm
'------------------------------------------------------------------

Dim Temp_a As Word
Dim Temp_b As Integer
Dim Temp_c As Byte

'------------------------------------------------------------------
' TestPins fr Zeitmessungen                     nur zu Testzwecken
'------------------------------------------------------------------
Config Portb.0 = Output
Config Portb.1 = Output


'------------------------------------------------------------------
' Hauptmenue
'------------------------------------------------------------------
_kalibrieren Alias 1 + 48 + 128
_texteingabe Alias 2 + 48 + 128
_delete Alias 3 + 48 + 128
_touch_werte_anzeigen Alias 4 + 48 + 128
_menue Alias 5 + 48 + 128
_timers Alias 6 + 48 + 128
_zeichensatz_anzeigen Alias 7 + 48 + 128


Gosub Auswahl



Do

If Touch_meldet = Ja Then
   Touch_meldet = Erledigt

   Select Case F_taste
      Case _kalibrieren : Gosub Touch_kalibrieren
      Case _texteingabe : Gosub Zeicheneingabe
      Case _touch_werte_anzeigen : Gosub Touch_werte_anzeigen
'      Case _timers : Gosub Timers
      Case _zeichensatz_anzeigen : Gosub Zeichensatz_anzeigen
   End Select

End If
Loop

'------------------------------------------------------------------
Auswahl:
'------------------------------------------------------------------

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 11
Locate_spalte = 0
Lcd_string = "     A u s w a h l"
Gosub Lcd_write_string

Locate_zeile = 31
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "ZEICHENSATZ  TOUCHWERTE"
Lcd_form_3 = "(7        )  (4       )"
Gosub Lcd_write_string

'Locate_zeile = 51
'Locate_spalte = 0
'Muster    = "01234567890123456789012"
'Lcd_string = "  TOUCHWERTE ANZEIGEN"
'Lcd_form_5 = "  (4                )"
'Gosub Lcd_write_string

Locate_zeile = 71
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "KALIBRIEREN TEXTEINGABE"
Lcd_form_7 = "(1        ) (2        )"
Gosub Lcd_write_string

Gosub Lcd_form_aufbau
Return


'------------------------------------------------------------------
Zeichensatz_anzeigen:
'------------------------------------------------------------------

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 36
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "                   MEN" + Chr(&H5d)           '
Lcd_form_3 = "                   (5 )"
Lcd_form_4 = "                   (5 )"
Gosub Lcd_write_string
Gosub Lcd_form_aufbau

Temp_c = 0

For Temp_b = 0 To 70 Step 10
   Locate_zeile = Temp_b
   Locate_spalte = 1

   For Temp_a = 0 To 15
      Lcd_string = Chr(temp_c)
      Incr Temp_c
      Gosub Lcd_write_string
   Next Temp_a

Next Temp_b

Do
Loop Until F_taste = _menue
Gosub Auswahl

Return





'------------------------------------------------------------------
Touch_werte_anzeigen:
'------------------------------------------------------------------
Set Portb.1

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 01
Locate_spalte = 0
Lcd_string = "01234567890123456789012"
Lcd_form_0 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 11
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "1                     " + Chr(&H5b)           '
Lcd_form_1 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 21
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "2                     " + Chr(&H5c)           '
Lcd_form_2 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 31
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "3                     " + Chr(&H5d)           '
Lcd_form_3 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 41
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "4        MENUE        " + Chr(&H7b)           '
Lcd_form_4 = "4        (5 )         " + Chr(&H7b)           '
Gosub Lcd_write_string

Locate_zeile = 51
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "5                     " + Chr(&H7c)           '
Lcd_form_5 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 61
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "6                     " + Chr(&H7d)           '
Lcd_form_6 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 71
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "7abcdefghijklmnopqrstu" + Chr(&H7e)           '
Lcd_form_7 = Lcd_string
Gosub Lcd_write_string

Gosub Lcd_form_aufbau


Locate_zeile = 51
Locate_spalte = 2
Lcd_string = "X1P=" + Str(x_1_pos) + "  Y1P=" + Str(y_1_pos)
Gosub Lcd_write_string

Locate_zeile = 61
Locate_spalte = 2
Lcd_string = "XpS=" + Str(x_pro_spaltenbreite) + " YpZ=" + Str(y_pro_zeilenhoehe)
Gosub Lcd_write_string


Reset Portb.1


Do

'------------------------------------------------------------------
' Touch Berhrung anzeigen
'------------------------------------------------------------------

      Locate_zeile = 25
      Locate_spalte = 19
      If Touch_aktiviert > 1 Then
         Lcd_string = Chr(7)
      Else
         Lcd_string = Chr(6)
      End If
      Gosub Lcd_write_string


   If Touch_meldet = Ja Then

      Locate_zeile = 21
      Locate_spalte = 2
      Lcd_string = "Raster:Y=" + Str(y_melden) + " X=" + Str(x_melden) + "    "
      Gosub Lcd_write_string


      Locate_zeile = 31
      Locate_spalte = 2
      Lcd_string = "Fein:Y=" + Str(y_aktuell) + " X=" + Str(x_aktuell) + "    "
      Gosub Lcd_write_string

      Lcd_string = "Chr:" + Chr(f_taste) + " Wert=" + Str(f_taste) + "    "
      Locate_zeile = 11
      Locate_spalte = 2
      Gosub Lcd_write_string

      Touch_meldet = Erledigt
End If

Loop Until F_taste = _menue
Gosub Auswahl
Return




'------------------------------------------------------------------
Zeicheneingabe:
'------------------------------------------------------------------

Set Portb.1

Gosub Lcd_clear
Gosub Lcd_form_clear

Locate_zeile = 21
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "  ABCDEFGHIJKLM   123 "
Lcd_form_2 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 31
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "  NOPQRSTUVWXYZ   456"
Lcd_form_3 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 41
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "  abcdefghijklm   789"
Lcd_form_4 = Lcd_string
Gosub Lcd_write_string

Locate_zeile = 51
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "  nopqrstuvwxyz   0 _"
Lcd_form_5 = Lcd_string
Gosub Lcd_write_string


Locate_zeile = 71
Locate_spalte = 0
'Muster    = "01234567890123456789012"
Lcd_string = "DELETE            MENUE"
Lcd_form_7 = "(3   )            (5  )"
Gosub Lcd_write_string
Gosub Lcd_form_aufbau

Touch_string = " "
Touch_string_zeiger = 1

Locate_zeile = 61
Locate_spalte = 0
Lcd_string = "Input:" + Touch_string + "-"
Gosub Lcd_write_string

Reset Portb.1



Do
If Touch_meldet = Ja Then

Set Portb.1

   If F_taste < 128 Then
      Lcd_string = "Zeichen :" + Chr(f_taste) + " Wert=" + Str(f_taste) + "    "
   Else
      Lcd_string = "Funktion:" + Chr(f_taste) + " Wert=" + Str(f_taste) + "    "
   End If
   Locate_zeile = 11
   Locate_spalte = 0
   Gosub Lcd_write_string


   Locate_zeile = 01
   Locate_spalte = 0
   Lcd_string = "Position:  Y=" + Str(y_melden) + "  X=" + Str(x_melden) + "   "
   Gosub Lcd_write_string


'------------------------------------------------------------------
' Eingabetext aufbauen
'------------------------------------------------------------------
   If F_taste > 1 And F_taste < 128 And Touch_string_zeiger < 13 Then
         Touch_string = Touch_string + Chr(f_taste)
         Incr Touch_string_zeiger
         Locate_zeile = 61
         Locate_spalte = 0
         If Touch_string_zeiger > 12 Then
            Lcd_string = "Input:" + Touch_string
         Else
         Lcd_string = "Input:" + Touch_string + "-"
         End If
         Gosub Lcd_write_string
   End If

   If F_taste = _delete Then
      If Touch_string_zeiger > 1 Then
            Decr Touch_string_zeiger
            Touch_string = Left(touch_string , Touch_string_zeiger)
            Locate_zeile = 61
            Locate_spalte = 0
            Lcd_string = "Input:" + Touch_string + "- "
            Gosub Lcd_write_string
      End If
    End If


   Touch_meldet = Erledigt

   Reset Portb.1

End If

Loop Until F_taste = _menue
Gosub Auswahl
Return



'------------------------------------------------------------------
'Unterprogramme
'------------------------------------------------------------------

'------------------------------------------------------------------
Display_und_touch_initialisieren:
'------------------------------------------------------------------
   ' Display und Touch initialisieren
   ' GLcd im Grafikmode mit 23 Spalten initialisieren:
   Waitms 50

   Gosub Reset_an_display_senden
   Waitms 10

   Gosub Lcd_initialisieren
   Gosub Lcd_clear
   Gosub Touch_init

   'Touch per Interupt abfragen
   Config Timer0 = Timer , Prescale = 1024
   Enable Timer0
   Enable Interrupts
   On Ovf0 Tim0_isr                                         'enable the interrupt

'    Gosub ' Touch_kal_werte_laden
'  Return


'------------------------------------------------------------------
' Touch_kal_werte_laden:
'------------------------------------------------------------------
         Readeeprom Temp_a , Touch_kal_werte
   If Temp_a = Kalibriert Then
         Readeeprom X_1_pos
         Readeeprom Y_1_pos
         Readeeprom X_pro_spaltenbreite
         Readeeprom Y_pro_zeilenhoehe
   Else
         Gosub Touch_kalibrieren
   End If
Return


'------------------------------------------------------------------
Touch_kalibrieren:
'------------------------------------------------------------------
Gosub Lcd_clear
Gosub Lcd_form_clear
Gosub Zielscheiben_malen
   If X_pro_spaltenbreite < 114 Or X_pro_spaltenbreite > 130 Then
         Gosub Touch_default_werte_eintragen
   Elseif Y_pro_zeilenhoehe < 79 Or Y_pro_zeilenhoehe > 91 Then
         Gosub Touch_default_werte_eintragen
   Elseif X_1_pos > 80 Or Y_1_pos > 80 Then
         Gosub Touch_default_werte_eintragen
   Else
         Temp_a = Kalibriert
         Writeeeprom Temp_a , Touch_kal_werte
         Writeeeprom X_1_pos
         Writeeeprom Y_1_pos
         Writeeeprom X_pro_spaltenbreite
         Writeeeprom Y_pro_zeilenhoehe
   End If
   Gosub Auswahl
Return


'------------------------------------------------------------------
Touch_default_werte_eintragen:
'------------------------------------------------------------------
   Locate_zeile = 51
   Locate_spalte = 1
   Lcd_string = "KX=" + Str(x_pro_spaltenbreite) + " "
   Gosub Lcd_write_string

   Lcd_string = "KY=" + Str(y_pro_zeilenhoehe)
   Gosub Lcd_write_string
   Lcd_string = " Fehler"
   Gosub Lcd_write_string

   X_pro_spaltenbreite = 122
   Y_pro_zeilenhoehe = 85
   X_1_pos = 50
   Y_1_pos = 60

   Wait 5

Return

'------------------------------------------------------------------
Zielscheiben_malen:
'------------------------------------------------------------------

   'Muster    = "01234567890123456789012"
   Lcd_string = "     Zum Kalibrieren"
   Locate_zeile = 31
   Locate_spalte = 0
   Gosub Lcd_write_string

   'Muster    = "01234567890123456789012"
   Lcd_string = "     Kreuze antippen"
   Locate_zeile = 45
   Locate_spalte = 0
   Gosub Lcd_write_string

   Wait 1

   Lcd_string = Chr(1) + Chr(2)
   Locate_zeile = 06
   Locate_spalte = 0
   Gosub Touch_beruehrung_abwarten
   X_oben_links = X_aktuell
   Y_oben_links = Y_aktuell

   Locate_zeile = 06
   Locate_spalte = 21
   Gosub Touch_beruehrung_abwarten
   X_oben_rechts = X_aktuell
   Y_oben_rechts = Y_aktuell

   Locate_zeile = 66
   Locate_spalte = 0
   Gosub Touch_beruehrung_abwarten
   X_unten_links = X_aktuell
   Y_unten_links = Y_aktuell

   Locate_zeile = 66
   Locate_spalte = 21
   Gosub Touch_beruehrung_abwarten
   X_unten_rechts = X_aktuell
   Y_unten_rechts = Y_aktuell

   Temp_a = X_oben_links + X_unten_links
   Temp_a = Temp_a + 1
   Temp_a = Temp_a / 2
   X_1_pos = Temp_a

   Temp_a = Y_oben_rechts + Y_oben_links
   Temp_a = Temp_a + 1
   Temp_a = Temp_a / 2
   Y_1_pos = Temp_a

   Temp_a = X_oben_rechts - X_oben_links
   Temp_b = X_unten_rechts - X_unten_links
   Temp_a = Temp_a + Temp_b
   Temp_a = 43130 / Temp_a                                  '43008= 2*1024 *21 =rechnerisch, ohne runden
   X_pro_spaltenbreite = Temp_a

   Temp_a = Y_unten_rechts - Y_oben_rechts
   Temp_b = Y_unten_links - X_oben_links
   Temp_a = Temp_a + Temp_b
   Temp_a = 26000 / Temp_a                                  '24567= 2*2048 *6  =rechnerisch, ohne runden
   Y_pro_zeilenhoehe = Temp_a
Return


'------------------------------------------------------------------
Touch_beruehrung_abwarten:
'------------------------------------------------------------------
   Gosub Lcd_write_string                                   'Kreuzchen malen

   Do
   Loop Until Touch_meldet = Ja
   Touch_meldet = Erledigt

   Gosub Lcd_clear
Return



'-------------------------- HINWEIS -------------------------------
' PB.0 und PB.1 werden verwendet, dienen aber nur zu Zeit-Messungen.
' Wenn bentigt enfach aus dem Programm lschen.
'------------------------------------------------------------------
Tim0_isr:
'------------------------------------------------------------------
     Set Portb.0                                            ' fr Zeitmessung

Incr Irq_teiler
If Irq_teiler = 2 Then                                      ' Vorteiler durch 2 fr !6 Mhz
                                                             ' ergibt Touchabfrage alle 33 msec
    Irq_teiler = 0
    Gosub Touch_auswerten

   Incr Ticker
   If Ticker = 3 Then
      Ticker = 0
      If Timer_a > 0 Then Decr Timer_a
      If Timer_b > 0 Then Decr Timer_b
   End If
End If

   Reset Portb.0                                            ' fr Zeitmessung
Return


'------------------------------------------------------------------
' und hier gehts in Assembler weiter ....
'------------------------------------------------------------------

$include "Touch16080.txt"
$include "LCD16080.txt"

End

$include "RomTabs.txt"